unit UTestCases;

interface

uses
  Classes, SysUtils, UTestCase, UEngine;

type

  { TTestCaseRun }

  TTestCaseRun = class(TTestCase)
    InitTrackCount: Integer;
    Engine: TEngine;
    procedure Run; override;
    constructor Create; override;
    destructor Destroy; override;
  end;

function InitTestCases: TTestCases;


implementation

function InitTestCases: TTestCases;
begin
  Result := TTestCases.Create;
  with Result do begin
    AddNew('Tick', TTestCaseRun);
    with TTestCaseRun(AddNew('One track', TTestCaseRun)) do begin
      InitTrackCount := 1;
    end;
  end;
end;

{ TTestCaseRun }

procedure TTestCaseRun.Run;
var
  I: Integer;
  MetroLine: TMetroLine;
  UsedLines: Integer;
begin
  Engine.Map.Size := Point(640, 480);
  Engine.View.DestRect := Rect(0, 0, 640, 480);
  Engine.NewGame;
  if InitTrackCount > 0 then begin
    for I := 0 to InitTrackCount - 1 do begin
      MetroLine := Engine.GetSelectedOrUnusedMetroLine;
      MetroLine.ConnectStation(Engine.Stations[0], nil, nil);
      MetroLine.ConnectStation(Engine.Stations[1], MetroLine.LineStations.Last, nil);
    end;
    UsedLines := 0;
    for I := 0 to Engine.Lines.Count - 1 do
      if Engine.Lines[I].LineStations.Count > 0 then Inc(UsedLines);
    Evaluate(UsedLines = InitTrackCount);
    Exit;
  end;
  for I := 0 to 100 do
    Engine.Tick;
  Pass;
end;

constructor TTestCaseRun.Create;
begin
  inherited;
  Engine := TEngine.Create(nil);
end;

destructor TTestCaseRun.Destroy;
begin
  FreeAndNil(Engine);
end;

end.

