unit UView;

interface

uses
  Classes, SysUtils;

type
  { TView }

  TView = class
  private
    FDestRect: TRect;
    FSourceRect: TRect;
    FZoom: Double;
    procedure SetDestRect(AValue: TRect);
    procedure SetSourceRect(AValue: TRect);
    procedure SetZoom(AValue: Double);
  public
    procedure Assign(Source: TView);
    function PointDestToSrc(Pos: TPoint): TPoint;
    function PointSrcToDest(Pos: TPoint): TPoint;
    constructor Create;
    property SourceRect: TRect read FSourceRect write SetSourceRect;
    property DestRect: TRect read FDestRect write SetDestRect;
    property Zoom: Double read FZoom write SetZoom;
  end;


implementation

uses
  UGeometric;

resourcestring
  SZeroZoomNotAlowed = 'Zero zoom not allowed';

{ TView }

procedure TView.SetDestRect(AValue: TRect);
var
  Diff: TPoint;
begin
  if RectEquals(FDestRect, AValue) then Exit;
  Diff := Point(Trunc((DestRect.Right - DestRect.Left) / Zoom - (AValue.Right - AValue.Left) / Zoom) div 2,
    Trunc((DestRect.Bottom - DestRect.Top) / Zoom - (AValue.Bottom - AValue.Top) / Zoom) div 2);
  FDestRect := AValue;
  FSourceRect := Bounds(FSourceRect.Left + Diff.X, FSourceRect.Top + Diff.Y,
    Trunc((DestRect.Right - DestRect.Left) / Zoom),
    Trunc((DestRect.Bottom - DestRect.Top) / Zoom));
end;

procedure TView.SetSourceRect(AValue: TRect);
var
  ZX: Double;
  ZY: Double;
begin
  if RectEquals(FSourceRect, AValue) then Exit;
  FSourceRect := AValue;
  if ((FSourceRect.Right - FSourceRect.Left) <> 0) and
  ((FSourceRect.Bottom - FSourceRect.Top) <> 0) then begin
    ZX := (FDestRect.Right - FDestRect.Left) / (FSourceRect.Right - FSourceRect.Left);
    ZY := (FDestRect.Bottom - FDestRect.Top) / (FSourceRect.Bottom - FSourceRect.Top);
    if ZX > ZY then
      Zoom := ZY
      else Zoom := ZX;
  end else Zoom := 1;
end;

procedure TView.SetZoom(AValue: Double);
begin
  if FZoom = AValue then Exit;
  if AValue = 0 then
    raise Exception.Create(SZeroZoomNotAlowed);
  FZoom := AValue;
  FSourceRect := Bounds(Trunc(FSourceRect.Left + (FSourceRect.Right - FSourceRect.Left) div 2 - (DestRect.Right - DestRect.Left) / Zoom / 2),
    Trunc(FSourceRect.Top +  (FSourceRect.Bottom - FSourceRect.Top) div 2 - (FDestRect.Bottom - DestRect.Top) / Zoom / 2),
    Trunc((DestRect.Right - DestRect.Left) / Zoom),
    Trunc((DestRect.Bottom - DestRect.Top) / Zoom));
end;

procedure TView.Assign(Source: TView);
begin
  FDestRect := Source.FDestRect;
  FSourceRect := Source.FSourceRect;
  FZoom := Source.FZoom;
end;

function TView.PointDestToSrc(Pos: TPoint): TPoint;
begin
  Result := Point(Trunc(Pos.X / FZoom + FSourceRect.Left),
    Trunc(Pos.Y / FZoom + FSourceRect.Top));
end;

function TView.PointSrcToDest(Pos: TPoint): TPoint;
begin
  Result := Point(Trunc((Pos.X - FSourceRect.Left) * FZoom),
    Trunc((Pos.Y - FSourceRect.Top) * FZoom));
end;

constructor TView.Create;
begin
  Zoom := 1;
end;

end.

